@@ -51,6 +51,11 @@ WnckWindow* _wnck_window_create  (Window      xwindow,
                                   WnckScreen *screen);
 void        _wnck_window_destroy (WnckWindow *window);
 
+const char* _wnck_window_get_startup_id (WnckWindow *window);
+
+const char* _wnck_window_get_resource_class (WnckWindow *window);
+const char* _wnck_window_get_resource_name  (WnckWindow *window);
+
 WnckWorkspace* _wnck_workspace_create  (int            number,
 					WnckScreen    *screen);
 void           _wnck_workspace_destroy (WnckWorkspace *space);
@@ -68,6 +73,12 @@ void             _wnck_application_destroy (WnckApplication *app);
 
 void _wnck_workspace_update_name (WnckWorkspace *workspace,
                                   const char    *name);
+void _wnck_screen_change_workspace_name (WnckScreen *screen,
+                                         int         number,
+                                         const char *name);
+
+gboolean _wnck_workspace_set_geometry (WnckWorkspace *space, int w, int h);
+gboolean _wnck_workspace_set_viewport (WnckWorkspace *space, int x, int y);
 
 void _wnck_init (void);
 
