@@ -2,6 +2,8 @@
 
 /*
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2003 Kim Woelders
+ * Copyright (C) 2003 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,7 +23,6 @@
 
 #include <string.h>
 #include "window.h"
-#include "class-group.h"
 #include "xutils.h"
 #include "private.h"
 #include "wnck-enum-types.h"
@@ -43,14 +44,14 @@ static GHashTable *window_hash = NULL;
     ((window)->priv->skip_pager          << 4) |        \
     ((window)->priv->skip_taskbar        << 5) |        \
     ((window)->priv->is_sticky           << 6) |        \
-    ((window)->priv->is_hidden           << 7) )
+    ((window)->priv->is_hidden           << 7) |        \
+    ((window)->priv->is_fullscreen       << 8) )
 
 struct _WnckWindowPrivate
 {
   Window xwindow;
   WnckScreen *screen;
   WnckApplication *app;
-  WnckClassGroup *class_group;
   Window group_leader;
   Window transient_for;
   char *name;
@@ -93,6 +94,7 @@ struct _WnckWindowPrivate
   guint skip_taskbar : 1;
   guint is_sticky : 1;
   guint is_hidden : 1;
+  guint is_fullscreen : 1;
 
   /* _NET_WM_STATE_HIDDEN doesn't map directly into an
    * externally-visible state (it determines the WM_STATE
@@ -477,14 +479,6 @@ wnck_window_get_xid (WnckWindow *window)
   return window->priv->xwindow;
 }
 
-WnckClassGroup *
-wnck_window_get_class_group (WnckWindow *window)
-{
-  g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
-
-  return window->priv->class_group;
-}
-
 /**
  * wnck_window_get_session_id:
  * @window: a #WnckWindow
@@ -689,6 +683,14 @@ wnck_window_is_skip_tasklist          (WnckWindow *window)
   return window->priv->skip_taskbar;
 }
 
+gboolean
+wnck_window_is_fullscreen                 (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
+
+  return window->priv->is_fullscreen;
+}
+
 void
 wnck_window_set_skip_tasklist (WnckWindow *window,
                                gboolean skip)
@@ -701,6 +703,18 @@ wnck_window_set_skip_tasklist (WnckWindow *window,
                       0);
 }
 
+void
+wnck_window_set_fullscreen (WnckWindow *window,
+                               gboolean fullscreen)
+{
+  g_return_if_fail (WNCK_IS_WINDOW (window));
+  _wnck_change_state (WNCK_SCREEN_XSCREEN (window->priv->screen),
+		      window->priv->xwindow,
+                      fullscreen,
+                      _wnck_atom_get ("_NET_WM_STATE_FULLSCREEN"),
+                      0);
+}
+
 /**
  * wnck_window_is_sticky:
  * @window: a #WnckWindow
@@ -1294,6 +1308,43 @@ wnck_window_is_on_workspace (WnckWindow    *window,
     wnck_window_get_workspace (window) == workspace;
 }
 
+/**
+ * wnck_window_is_in_viewport:
+ * @window: a #WnckWindow
+ * @workspace: a #WnckWorkspace
+ * 
+ * Returns #TRUE if the window is inside the current viewport
+ * of the given workspace.
+ * 
+ * Return value: %TRUE if @window appears in current viewport
+ **/
+gboolean
+wnck_window_is_in_viewport (WnckWindow    *window,
+                            WnckWorkspace *workspace)
+{
+  GdkRectangle window_rect;
+  GdkRectangle viewport_rect;
+  
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
+  g_return_val_if_fail (WNCK_IS_WORKSPACE (workspace), FALSE);
+
+  if (wnck_window_get_workspace (window) != workspace &&
+      !wnck_window_is_pinned (window))
+    return FALSE;
+
+  viewport_rect.x = wnck_workspace_get_viewport_x (workspace);
+  viewport_rect.y = wnck_workspace_get_viewport_y (workspace);
+  viewport_rect.width = wnck_screen_get_width (window->priv->screen);
+  viewport_rect.height = wnck_screen_get_height (window->priv->screen);
+
+  window_rect.x = window->priv->x + viewport_rect.x;
+  window_rect.y = window->priv->y + viewport_rect.y;
+  window_rect.width = window->priv->width;
+  window_rect.height = window->priv->height;
+
+  return gdk_rectangle_intersect (&viewport_rect, &window_rect, &window_rect);
+}
+
 void
 _wnck_window_set_application (WnckWindow      *window,
                               WnckApplication *app)
@@ -1308,20 +1359,6 @@ _wnck_window_set_application (WnckWindow      *window,
   window->priv->app = app;
 }
 
-void
-_wnck_window_set_class_group (WnckWindow     *window,
-			      WnckClassGroup *class_group)
-{
-  g_return_if_fail (WNCK_IS_WINDOW (window));
-  g_return_if_fail (class_group == NULL || WNCK_IS_CLASS_GROUP (class_group));
-
-  if (class_group)
-    g_object_ref (G_OBJECT (class_group));
-  if (window->priv->class_group)
-    g_object_unref (G_OBJECT (window->priv->class_group));
-  window->priv->class_group = class_group;
-}
-
 void
 _wnck_window_process_property_notify (WnckWindow *window,
                                       XEvent     *xevent)
@@ -1496,6 +1533,8 @@ update_state (WnckWindow *window)
             window->priv->is_sticky = TRUE;
           else if (atoms[i] == _wnck_atom_get ("_NET_WM_STATE_SHADED"))
             window->priv->is_shaded = TRUE;
+          else if (atoms[i] == _wnck_atom_get ("_NET_WM_STATE_FULLSCREEN"))
+            window->priv->is_fullscreen = TRUE;
           else if (atoms[i] == _wnck_atom_get ("_NET_WM_STATE_SKIP_TASKBAR"))
             window->priv->skip_taskbar = TRUE;
           else if (atoms[i] == _wnck_atom_get ("_NET_WM_STATE_SKIP_PAGER"))
