@@ -2,6 +2,8 @@
 
 /*
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2003 Kim Woelders
+ * Copyright (C) 2003 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,12 +28,13 @@
 #include "private.h"
 #include <string.h>
 
-static GHashTable* workspace_hash = NULL;
-
 struct _WnckWorkspacePrivate
 {
+  WnckScreen *screen;
   int number;
   char *name;
+  int width, height;            /* Workspace size */
+  int viewport_x, viewport_y;   /* Viewport origin */
 };
 
 enum {
@@ -120,27 +123,6 @@ wnck_workspace_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-/**
- * wnck_workspace_get:
- * @number: a workspace index
- * 
- * Gets the workspace numbered @number, or returns %NULL if
- * no such workspace exists.
- * 
- * Return value: the workspace, or %NULL
- **/
-WnckWorkspace*
-wnck_workspace_get (int number)
-{
-  /* We trust this function with property-provided numbers, it
-   * must reliably return NULL on bad data
-   */
-  if (workspace_hash == NULL)
-    return NULL;
-  else
-    return g_hash_table_lookup (workspace_hash, &number);
-}
-
 /**
  * wnck_workspace_get_number:
  * @space: a #WnckWorkspace
@@ -165,9 +147,6 @@ wnck_workspace_get_number (WnckWorkspace *space)
  * in the user interface. If the user hasn't set a special name,
  * will be something like "Workspace 3" - otherwise whatever name
  * the user set.
- *
- * FIXME haven't actually implemented reading the user-set
- * name from the window manager.
  * 
  * Return value: workspace name, never %NULL 
  **/
@@ -179,6 +158,26 @@ wnck_workspace_get_name (WnckWorkspace *space)
   return space->priv->name;
 }
 
+/**
+ * wnck_workspace_change_name:
+ * @space: a #WnckWorkspace
+ * @name: new workspace name
+ *
+ * Try changing the name of the workspace.
+ * 
+ **/
+void
+wnck_workspace_change_name (WnckWorkspace *space,
+                            const char    *name)
+{
+  g_return_if_fail (WNCK_IS_WORKSPACE (space));
+  g_return_if_fail (name != NULL);
+
+  _wnck_screen_change_workspace_name (space->priv->screen,
+                                      space->priv->number,
+                                      name);
+}
+
 /**
  * wnck_workspace_activate:
  * @space: a #WnckWorkspace
@@ -191,48 +190,31 @@ wnck_workspace_activate (WnckWorkspace *space)
 {
   g_return_if_fail (WNCK_IS_WORKSPACE (space));
 
-  _wnck_activate_workspace (space->priv->number);
+  _wnck_activate_workspace (WNCK_SCREEN_XSCREEN (space->priv->screen), space->priv->number);
 }
 
 WnckWorkspace*
-_wnck_workspace_create  (int number)
+_wnck_workspace_create (int number, WnckScreen *screen)
 {
   WnckWorkspace *space;
   
-  if (workspace_hash == NULL)
-    workspace_hash = g_hash_table_new (g_int_hash, g_int_equal);
-
-  g_return_val_if_fail (g_hash_table_lookup (workspace_hash, &number) == NULL,
-                        NULL);
-  
   space = g_object_new (WNCK_TYPE_WORKSPACE, NULL);
   space->priv->number = number;
   space->priv->name = NULL;
+  space->priv->screen = screen;
 
   _wnck_workspace_update_name (space, NULL);
   
-  g_hash_table_insert (workspace_hash, &space->priv->number, space);
+  /* Just set reasonable defaults */
+  space->priv->width = wnck_screen_get_width (screen);
+  space->priv->height = wnck_screen_get_height (screen);
+
+  space->priv->viewport_x = 0;
+  space->priv->viewport_y = 0;
 
-  /* Hash now owns one ref, caller gets none */
-  
   return space;
 }
 
-void
-_wnck_workspace_destroy (WnckWorkspace *space)
-{
-  g_return_if_fail (wnck_workspace_get (space->priv->number) == space);
-  
-  g_hash_table_remove (workspace_hash, &space->priv->number);
-
-  g_return_if_fail (wnck_workspace_get (space->priv->number) == NULL);
-
-  space->priv->number = -1;
-  
-  /* remove hash's ref on the workspace */  
-  g_object_unref (G_OBJECT (space));
-}
-
 void
 _wnck_workspace_update_name (WnckWorkspace *space,
                              const char    *name)
@@ -263,3 +245,59 @@ emit_name_changed (WnckWorkspace *space)
                  signals[NAME_CHANGED],
                  0);
 }
+
+gboolean
+_wnck_workspace_set_geometry (WnckWorkspace *space,
+                              int            w,
+                              int            h)
+{
+  if (space->priv->width != w || space->priv->height != h)
+    {
+      space->priv->width = w;
+      space->priv->height = h;
+
+      return TRUE;  /* change was made */
+    }
+  else
+    return FALSE; 
+}
+
+gboolean
+_wnck_workspace_set_viewport (WnckWorkspace *space,
+                              int            x,
+                              int            y)
+{
+  if (space->priv->viewport_x != x || space->priv->viewport_y != y)
+    {
+      space->priv->viewport_x = x;
+      space->priv->viewport_y = y;
+
+      return TRUE; /* change was made */
+    }
+  else
+    return FALSE;
+}
+
+int
+wnck_workspace_get_width (WnckWorkspace *space)
+{
+  return space->priv->width;
+}
+
+int
+wnck_workspace_get_height (WnckWorkspace *space)
+{
+  return space->priv->height;
+}
+
+int
+wnck_workspace_get_viewport_x (WnckWorkspace *space)
+{
+  return space->priv->viewport_x;
+}
+
+int
+wnck_workspace_get_viewport_y (WnckWorkspace *space)
+{
+  return space->priv->viewport_y;
+}
